import datetime
import re
from django.shortcuts import render
from django.contrib.auth.decorators import permission_required
import django.http
from . import models


@permission_required("geolist.can_view_geolist")
def view_landing(request):
    geolists = []
    for geolist in models.Geolist.objects.all():
        if not models.History.objects.filter(user=request.user, geolist=geolist).exists():
            history = models.History.objects.create(user=request.user, geolist=geolist)
        else:
            history = models.History.objects.get(user=request.user, geolist=geolist)
        geolist.history = history
        geolists.append(geolist)
    return render(request, "geolist/landing.html", {
        "geolists": geolists
    })


@permission_required("geolist.can_change_geolist")
def view_list(request, slug):
    query = models.Geolist.objects.filter(slug=slug)
    if not query.exists():
        raise django.http.Http404("Cette liste n'existe pas.")
    glist = query.get()

    if not models.History.objects.filter(user=request.user, geolist=glist).exists():
        history = models.History.objects.create(user=request.user, geolist=glist)
    else:
        history = models.History.objects.get(user=request.user, geolist=glist)

    if request.method == "POST":
        itemid = request.POST.get("itemid")
        visited = "visited" in request.POST
        date_visited = request.POST.get("date")
        if re.match(r"\d+\-\d+\-\d+", date_visited):
            date_visited = datetime.datetime.strptime(date_visited, "%Y-%m-%d")
        else:
            date_visited = None
        if models.GeolistItem.objects.filter(id=itemid).exists():
            item = models.GeolistItem.objects.get(id=itemid)
            if visited:
                if history.visits.filter(item=item).exists():
                    visit = history.visits.get(item=item)
                    visit.date_visited = date_visited
                    visit.save()
                else:
                    models.Visit.objects.create(
                        history=history,
                        item=item,
                        date_visited=date_visited
                    )
            else:
                history.visits.filter(item=item).delete()

    items = []
    total = 0
    visited = 0
    for item in glist.items.all():
        total += 1
        item.is_visited, _ = item.visited(request.user)
        if item.is_visited:
            visited += 1
        items.append(item)

    progress = 0 if total == 0 else visited / total * 100
    history.progress = progress
    history.save()

    return render(request, "geolist/geolist.html", {
        "geolist": glist,
        "items": items,
        "total": total,
        "visited": visited,
        "progress": progress
    })


@permission_required("geolist.can_view_geolistitem")
def view_points(request, slug):
    query = models.Geolist.objects.filter(slug=slug)
    if not query.exists():
        raise django.http.Http404("Cette liste n'existe pas.")
    glist = query.get()
    data = []
    for item in glist.items.all():
        visited, date_visited = item.visited(request.user)
        data.append({
            "id": item.id,
            "label": item.label,
            "latitude": item.latitude,
            "longitude": item.longitude,
            "elevation": item.elevation,
            "visited": visited,
            "date": date_visited.strftime("%Y-%m-%d") if date_visited is not None else None
        })
    return django.http.JsonResponse({
        "points": data
    }, safe=False)
